@@ -51,6 +51,11 @@
  * 1.24        Fixed a bug where SLI_MOSAIC_MODE_AVAILABLE attribute would
  *             report false positives via the GPU and X screen target types
  * 1.25        Added 3D_VISION_PRO_TRANSCEIVER TargetType
+ * 1.26        Added XNVCTRLQueryXXXAttributePermissions.
+ * 1.27        Added DISPLAY TargetType
+ * 1.28        Added NV_CTRL_CURRENT_METAMODE_ID: clients should use this
+ *             attribute to switch MetaModes, rather than pass the MetaMode ID
+ *             through the RRSetScreenConfig protocol request.
  */
 
 #ifndef __NVCONTROL_H
@@ -61,7 +66,7 @@
 #define NV_CONTROL_NAME "NV-CONTROL"
 
 #define NV_CONTROL_MAJOR 1
-#define NV_CONTROL_MINOR 25
+#define NV_CONTROL_MINOR 29
 
 #define X_nvCtrlQueryExtension                      0
 #define X_nvCtrlIsNv                                1
@@ -92,7 +97,13 @@
 #define X_nvCtrlQueryValidAttributeValues64         26
 #define X_nvCtrlQueryAttribute64                    27
 #define X_nvCtrlQueryValidStringAttributeValues     28
-#define X_nvCtrlLastRequest (X_nvCtrlQueryValidStringAttributeValues + 1)
+#define X_nvCtrlQueryAttributePermissions                29
+#define X_nvCtrlQueryStringAttributePermissions          30
+#define X_nvCtrlQueryBinaryDataAttributePermissions      31
+#define X_nvCtrlQueryStringOperationAttributePermissions 32
+#define X_nvCtrlBindWarpPixmapName                       33
+
+#define X_nvCtrlLastRequest (X_nvCtrlBindWarpPixmapName + 1)
 
 
 /* Define 32 bit floats */
@@ -337,6 +348,29 @@ typedef struct {
 #define sz_xnvCtrlQueryValidAttributeValues64Reply 48
 #define sz_xnvCtrlQueryValidAttributeValues64Reply_extra ((48 - 32) >> 2)
 
+typedef struct {
+    CARD8 reqType;
+    CARD8 nvReqType;
+    CARD16 length B16;
+    CARD32 attribute B32;
+} xnvCtrlQueryAttributePermissionsReq;
+#define sz_xnvCtrlQueryAttributePermissionsReq 8
+
+typedef struct {
+    BYTE type;
+    BYTE pad0;
+    CARD16 sequenceNumber B16;
+    CARD32 length B32;
+    CARD32 flags B32;
+    INT32 attr_type B32;
+    CARD32 perms B32;
+    CARD32 pad5 B32;
+    CARD32 pad6 B32;
+    CARD32 pad7 B32;
+    CARD32 pad8 B32;
+} xnvCtrlQueryAttributePermissionsReply;
+#define sz_xnvCtrlQueryAttributePermissionsReply 32
+
 /* Set GVO Color Conversion request (deprecated) */
 typedef struct {
     CARD8 reqType;
@@ -499,6 +533,18 @@ typedef struct {
 } xnvCtrlStringOperationReply;
 #define sz_xnvCtrlStringOperationReply 32
 
+typedef struct {
+    CARD8 reqType;
+    CARD8 nvReqType;
+    CARD16 length B16;
+    CARD32 screen B32;
+    CARD32 pixmap B32;
+    CARD32 num_bytes B32; /* Length of string */
+    CARD32 dataType B32;
+    CARD32 vertexCount B32;
+} xnvCtrlBindWarpPixmapNameReq;
+#define sz_xnvCtrlBindWarpPixmapNameReq 24
+
 typedef struct {
     union {
         struct {
